import { Draft04 } from "json-schema-library";
import { jsonPointerForPosition } from "./utils/jsonPointers";
import { joinWithOr } from "./utils/formatting";
import getSchema from "./utils/schema-lib/getSchema";
/**
 * provides a JSON schema enabled tooltip extension for codemirror
 * @group Codemirror Extensions
 */
export function jsonSchemaHover(schema, options) {
    const hover = new JSONHover(schema, options);
    return async function jsonDoHover(view, pos, side) {
        return hover.doHover(view, pos, side);
    };
}
function formatComplexType(schema, type, draft) {
    return `${type}: ${joinWithOr(schema[type].map((s) => {
        var _a;
        return (_a = s.type) !== null && _a !== void 0 ? _a : draft.resolveRef(s).type;
    }))}`;
}
export class JSONHover {
    constructor(schema, opts) {
        this.opts = opts;
        this.schema = new Draft04(schema);
    }
    getDataForCursor(view, pos, side) {
        var _a;
        const pointer = jsonPointerForPosition(view.state, pos, side, (_a = this.opts) === null || _a === void 0 ? void 0 : _a.mode);
        if (!pointer) {
            return null;
        }
        const subSchema = getSchema(this.schema, pointer);
        // doesn't work for oneOf - we need to give it a shape to tell it what kind of schema type to return
        if (subSchema.type === "error") {
            return { pointer };
        }
        return { schema: subSchema, pointer };
    }
    formatMessage(data, draft) {
        const { schema } = data;
        const hoverWrapper = document.createElement("div");
        hoverWrapper.classList.add("cm6-json-schema-hover");
        const codeBlock = document.createElement("code");
        if (schema.type) {
            codeBlock.innerText = Array.isArray(schema.type)
                ? joinWithOr(schema.type)
                : schema.type;
        }
        if (schema.oneOf) {
            codeBlock.innerText = formatComplexType(schema, "oneOf", draft);
        }
        if (schema.allOf) {
            codeBlock.innerText = formatComplexType(schema, "anyOf", draft);
        }
        if (schema.allOf) {
            codeBlock.innerText = formatComplexType(schema, "allOf", draft);
        }
        if (schema.description) {
            const descriptionBlock = document.createElement("div");
            const codeWrapper = document.createElement("div");
            descriptionBlock.innerText = schema.description;
            hoverWrapper.appendChild(descriptionBlock);
            codeWrapper.appendChild(codeBlock);
            hoverWrapper.appendChild(codeWrapper);
            return hoverWrapper;
        }
        else {
            hoverWrapper.appendChild(codeBlock);
        }
        return hoverWrapper;
    }
    // return hover state for the current json schema property
    async doHover(view, pos, side) {
        var _a, _b;
        let start = pos, end = pos;
        try {
            const cursorData = this.getDataForCursor(view, pos, side);
            if (!(cursorData === null || cursorData === void 0 ? void 0 : cursorData.schema)) {
                return null;
            }
            // allow users to override the hover
            const formatter = (_b = (_a = this.opts) === null || _a === void 0 ? void 0 : _a.formatHover) !== null && _b !== void 0 ? _b : this.formatMessage;
            return {
                pos: start,
                end,
                above: true,
                arrow: true,
                create: (view) => {
                    return {
                        dom: formatter(cursorData, this.schema),
                    };
                },
            };
        }
        catch (_c) {
            return null;
        }
    }
}
